@@ -14,8 +14,11 @@
 #include "bgpq3.h"
 #include "sx_report.h"
 
+extern int debug_expander;
+
 int bgpq3_print_json_aspath(FILE* f, struct bgpq_expander* b);
 int bgpq3_print_bird_aspath(FILE* f, struct bgpq_expander* b);
+int bgpq3_print_openbgpd_aspath(FILE* f, struct bgpq_expander* b);
 
 int
 bgpq3_print_cisco_aspath(FILE* f, struct bgpq_expander* b)
@@ -78,6 +81,45 @@ bgpq3_print_cisco_aspath(FILE* f, struct bgpq_expander* b)
 		fprintf(f,"ip as-path access-list %s deny .*\n", b->name?b->name:"NN");
 	return 0;
 };
+
+int
+bgpq3_print_cisco_xr_aspath(FILE* f, struct bgpq_expander* b)
+{
+	int nc=0, i, j, k, comma=0;
+	fprintf(f, "as-path-set %s", b->name?b->name:"NN");
+	if(b->asn32s[b->asnumber/65536] &&
+		b->asn32s[b->asnumber/65536][(b->asnumber%65536)/8]&
+		(0x80>>(b->asnumber%8))) {
+		fprintf(f,"\n  ios-regex '^%u(_%u)*$'", b->asnumber,b->asnumber);
+		comma=1;
+	};
+	for(k=0;k<65536;k++) {
+		if(!b->asn32s[k]) continue;
+
+		for(i=0;i<8192;i++) {
+			for(j=0;j<8;j++) {
+				if(b->asn32s[k][i]&(0x80>>j)) {
+					if(k*65536+i*8+j==b->asnumber) continue;
+					if(!nc) {
+						fprintf(f,"%s\n  ios-regex '^%u(_[0-9]+)*_(%u",
+							comma?",":"", b->asnumber,k*65536+i*8+j);
+						comma=1;
+					} else {
+						fprintf(f,"|%u",k*65536+i*8+j);
+					}
+					nc++;
+					if(nc==b->aswidth) {
+						fprintf(f,")$'");
+						nc=0;
+					};
+				};
+			};
+		};
+	};
+	if(nc) fprintf(f,")$'");
+	fprintf(f,"\nend-set\n");
+	return 0;
+};
 int
 bgpq3_print_cisco_oaspath(FILE* f, struct bgpq_expander* b)
 {
@@ -136,6 +178,44 @@ bgpq3_print_cisco_oaspath(FILE* f, struct bgpq_expander* b)
 		fprintf(f,"ip as-path access-list %s deny .*\n", b->name?b->name:"NN");
 	return 0;
 };
+
+int
+bgpq3_print_cisco_xr_oaspath(FILE* f, struct bgpq_expander* b)
+{
+	int nc=0, i, j, k, comma=0;
+	fprintf(f, "as-path-set %s", b->name?b->name:"NN");
+	if(b->asn32s[b->asnumber/65536] &&
+		b->asn32s[b->asnumber/65536][(b->asnumber%65536)/8]&
+			(0x80>>(b->asnumber%8))) {
+		fprintf(f,"\n  ios-regex '^(_%u)*$'",b->asnumber);
+		comma=1;
+	};
+	for(k=0;k<65536;k++) {
+		if(!b->asn32s[k]) continue;
+		for(i=0;i<8192;i++) {
+			for(j=0;j<8;j++) {
+				if(b->asn32s[k][i]&(0x80>>j)) {
+					if(k*65536+i*8+j==b->asnumber) continue;
+					if(!nc) {
+						fprintf(f,"%s\n  ios-regex '^(_[0-9]+)*_(%u",
+							comma?",":"", k*65536+i*8+j);
+						comma=1;
+					} else {
+						fprintf(f,"|%u",k*65536+i*8+j);
+					}
+					nc++;
+					if(nc==b->aswidth) {
+						fprintf(f,")$'");
+						nc=0;
+					};
+				};
+			};
+		};
+	};
+	if(nc) fprintf(f,")$'");
+	fprintf(f,"\nend-set\n");
+	return 0;
+};
 		
 int
 bgpq3_print_juniper_aspath(FILE* f, struct bgpq_expander* b)
@@ -224,6 +304,25 @@ bgpq3_print_juniper_oaspath(FILE* f, struct bgpq_expander* b)
 	return 0;
 };
 
+int
+bgpq3_print_openbgpd_oaspath(FILE* f, struct bgpq_expander* b)
+{
+	int i, j, k;
+
+	for(k=0;k<65536;k++) {
+		if(!b->asn32s[k]) continue;
+
+		for(i=0;i<8192;i++) {
+			for(j=0;j<8;j++) {
+				if(b->asn32s[k][i]&(0x80>>j)) {
+					fprintf(f, "%s%u%s%u\n", "allow to AS ", b->asnumber, " AS ", k*65536+i*8+j);
+				};
+			};
+		};
+	};
+	return 0;
+};
+
 int
 bgpq3_print_aspath(FILE* f, struct bgpq_expander* b)
 {
@@ -231,10 +330,14 @@ bgpq3_print_aspath(FILE* f, struct bgpq_expander* b)
 		return bgpq3_print_juniper_aspath(f,b);
 	} else if(b->vendor==V_CISCO) {
 		return bgpq3_print_cisco_aspath(f,b);
+	} else if(b->vendor==V_CISCO_XR) {
+		return bgpq3_print_cisco_xr_aspath(f,b);
 	} else if(b->vendor==V_JSON) {
 		return bgpq3_print_json_aspath(f,b);
 	} else if(b->vendor==V_BIRD) {
 		return bgpq3_print_bird_aspath(f,b);
+	} else if(b->vendor==V_OPENBGPD) {
+		return bgpq3_print_openbgpd_aspath(f,b);
 	} else {
 		sx_report(SX_FATAL,"Unknown vendor %i\n", b->vendor);
 	};
@@ -248,6 +351,10 @@ bgpq3_print_oaspath(FILE* f, struct bgpq_expander* b)
 		return bgpq3_print_juniper_oaspath(f,b);
 	} else if(b->vendor==V_CISCO) {
 		return bgpq3_print_cisco_oaspath(f,b);
+	} else if(b->vendor==V_CISCO_XR) {
+		return bgpq3_print_cisco_xr_oaspath(f,b);
+	} else if(b->vendor==V_OPENBGPD) {
+		return bgpq3_print_openbgpd_oaspath(f,b);
 	} else {
 		sx_report(SX_FATAL,"Unknown vendor %i\n", b->vendor);
 	};
@@ -357,7 +464,8 @@ int
 bgpq3_print_bird_aspath(FILE* f, struct bgpq_expander* b)
 {
 	int nc=0, i, j, k, empty=1;
-	fprintf(f, "%s = [", b->name?b->name:"NN");
+	char buffer[2048];
+	snprintf(buffer, sizeof(buffer), "%s = [", b->name?b->name:"NN");
 
 	for(k=0;k<65536;k++) {
 		if(!b->asn32s[k]) continue;
@@ -365,6 +473,9 @@ bgpq3_print_bird_aspath(FILE* f, struct bgpq_expander* b)
 		for(i=0;i<8192;i++) {
 			for(j=0;j<8;j++) {
 				if(b->asn32s[k][i]&(0x80>>j)) {
+					if(buffer[0])
+						fprintf(f, "%s", buffer);
+					buffer[0]=0;
 					if(!nc) {
 						fprintf(f, "%s%u", empty?"":",\n    ", k*65536+i*8+j);
 						empty = 0;
@@ -379,7 +490,55 @@ bgpq3_print_bird_aspath(FILE* f, struct bgpq_expander* b)
 			};
 		};
 	};
-	fprintf(f,"];\n");
+	if(!empty)
+		fprintf(f,"];\n");
+	return 0;
+};
+
+void
+bgpq3_print_openbgpd_prefix(struct sx_radix_node* n, void* ff)
+{
+	char prefix[128];
+	FILE* f=(FILE*)ff;
+	if(n->isGlue)
+		goto checkSon;
+	if(!f)
+		f=stdout;
+	sx_prefix_snprintf(&n->prefix, prefix, sizeof(prefix));
+	if (!n->isAggregate) {
+		fprintf(f, "\n\t%s \\", prefix);
+	} else if (n->aggregateLow == n->aggregateHi) {
+		fprintf(f, "\n\t%s prefixlen = %u \\",
+			prefix, n->aggregateHi);
+	} else if (n->aggregateLow > n->prefix.masklen) {
+		fprintf(f, "\n\t%s prefixlen %u - %u \\",
+			prefix, n->aggregateLow, n->aggregateHi);
+	} else {
+		fprintf(f, "\n\t%s prefixlen %u - %u \\",
+			prefix, n->prefix.masklen, n->aggregateHi);
+	};
+	needscomma=1;
+checkSon:
+	if(n->son)
+		bgpq3_print_openbgpd_prefix(n->son, ff);
+};
+
+int
+bgpq3_print_openbgpd_aspath(FILE* f, struct bgpq_expander* b)
+{
+	int i, j, k;
+
+	for(k=0;k<65536;k++) {
+		if(!b->asn32s[k]) continue;
+
+		for(i=0;i<8192;i++) {
+			for(j=0;j<8;j++) {
+				if(b->asn32s[k][i]&(0x80>>j)) {
+					fprintf(f, "%s%u%s%u\n", "allow from AS ", b->asnumber, " AS ", k*65536+i*8+j);
+				};
+			};
+		};
+	};
 	return 0;
 };
 
@@ -408,28 +567,32 @@ checkSon:
 		
 
 static char* bname=NULL;
+static int   seq=0;
 
 void
 bgpq3_print_cprefix(struct sx_radix_node* n, void* ff)
 {
-	char prefix[128];
+	char prefix[128], seqno[16]="";
 	FILE* f=(FILE*)ff;
 	if(!f) f=stdout;
 	if(n->isGlue) goto checkSon;
 	sx_prefix_snprintf(&n->prefix,prefix,sizeof(prefix));
+	if(seq)
+		snprintf(seqno, sizeof(seqno), " seq %i", seq++);
 	if(n->isAggregate) {
 		if(n->aggregateLow>n->prefix.masklen) {
-			fprintf(f,"%s prefix-list %s permit %s ge %u le %u\n",
-				n->prefix.family==AF_INET?"ip":"ipv6",bname?bname:"NN",prefix,
-				n->aggregateLow,n->aggregateHi);
+			fprintf(f,"%s prefix-list %s%s permit %s ge %u le %u\n",
+				n->prefix.family==AF_INET?"ip":"ipv6",bname?bname:"NN",seqno,
+				prefix,n->aggregateLow,n->aggregateHi);
 		} else {
-			fprintf(f,"%s prefix-list %s permit %s le %u\n",
-				n->prefix.family==AF_INET?"ip":"ipv6",bname?bname:"NN",prefix,
-				n->aggregateHi);
+			fprintf(f,"%s prefix-list %s%s permit %s le %u\n",
+				n->prefix.family==AF_INET?"ip":"ipv6",bname?bname:"NN",seqno,
+				prefix,n->aggregateHi);
 		};
 	} else {
-		fprintf(f,"%s prefix-list %s permit %s\n",
-			(n->prefix.family==AF_INET)?"ip":"ipv6",bname?bname:"NN",prefix);
+		fprintf(f,"%s prefix-list %s%s permit %s\n",
+			(n->prefix.family==AF_INET)?"ip":"ipv6",bname?bname:"NN",seqno,
+			prefix);
 	};
 checkSon:
 	if(n->son)
@@ -566,10 +729,37 @@ bgpq3_print_juniper_routefilter(FILE* f, struct bgpq_expander* b)
 	return 0;
 };
 
+
+int
+bgpq3_print_openbgpd_prefixlist(FILE* f, struct bgpq_expander* b)
+{
+	if (!sx_radix_tree_empty(b->tree)) {
+		if(b->name){
+			if(strcmp(b->name, "NN") != 0) {
+				fprintf(f, "%s=\"", b->name);
+			}
+		}
+		fprintf(f,"prefix { \\");
+		sx_radix_tree_foreach(b->tree,bgpq3_print_openbgpd_prefix,f);
+		fprintf(f, "\n\t}");
+		if(b->name){
+			if(strcmp(b->name, "NN") != 0) {
+				fprintf(f, "\"");
+			}
+		}
+		fprintf(f, "\n");
+	} else {
+		fprintf(f, "# generated prefix-list %s (AS %u) is empty\n", bname, b->asnumber);
+		fprintf(f, "deny from AS %u\n", b->asnumber);
+	};
+	return 0;
+};
+
 int
 bgpq3_print_cisco_prefixlist(FILE* f, struct bgpq_expander* b)
 {
 	bname=b->name ? b->name : "NN";
+	seq=b->sequence;
 	fprintf(f,"no %s prefix-list %s\n",
 		(b->family==AF_INET)?"ip":"ipv6",bname);
 	if (!sx_radix_tree_empty(b->tree)) {
@@ -606,10 +796,47 @@ bgpq3_print_json_prefixlist(FILE* f, struct bgpq_expander* b)
 int
 bgpq3_print_bird_prefixlist(FILE* f, struct bgpq_expander* b)
 {
-	fprintf(f,"%s = [",
-		b->name?b->name:"NN");
-	sx_radix_tree_foreach(b->tree,bgpq3_print_bird_prefix,f);
-	fprintf(f,"\n];\n");
+	if (!sx_radix_tree_empty(b->tree)) {
+		fprintf(f,"%s = [",
+			b->name?b->name:"NN");
+		sx_radix_tree_foreach(b->tree,bgpq3_print_bird_prefix,f);
+		fprintf(f,"\n];\n");
+	} else {
+		SX_DEBUG(debug_expander, "skip empty prefix-list in BIRD format\n");
+	};
+	return 0;
+};
+
+struct fpcbdata {
+	FILE* f;
+	struct bgpq_expander* b;
+};
+
+void
+bgpq3_print_format_prefix(struct sx_radix_node* n, void* ff)
+{
+	char prefix[128];
+	struct fpcbdata* fpc=(struct fpcbdata*)ff;
+	FILE* f=fpc->f;
+	struct bgpq_expander* b=fpc->b;
+	if(n->isGlue)
+		return;
+	if(!f)
+		f=stdout;
+	memset(prefix, 0, sizeof(prefix));
+	sx_prefix_snprintf_fmt(&n->prefix, prefix, sizeof(prefix),
+		b->name?b->name:"NN", b->format);
+	fprintf(f, "%s", prefix);
+};
+
+
+int
+bgpq3_print_format_prefixlist(FILE* f, struct bgpq_expander* b)
+{
+	struct fpcbdata ff = {.f=f, .b=b};
+	sx_radix_tree_foreach(b->tree,bgpq3_print_format_prefix,&ff);
+	if (strcmp(b->format+strlen(b->format-2), "\n"))
+		fprintf(f, "\n");
 	return 0;
 };
 
@@ -637,6 +864,8 @@ bgpq3_print_prefixlist(FILE* f, struct bgpq_expander* b)
 		case V_CISCO_XR: return bgpq3_print_ciscoxr_prefixlist(f,b);
 		case V_JSON: return bgpq3_print_json_prefixlist(f,b);
 		case V_BIRD: return bgpq3_print_bird_prefixlist(f,b);
+		case V_OPENBGPD: return bgpq3_print_openbgpd_prefixlist(f,b);
+		case V_FORMAT: return bgpq3_print_format_prefixlist(f,b);
 	};
 	return 0;
 };
@@ -650,6 +879,8 @@ bgpq3_print_eacl(FILE* f, struct bgpq_expander* b)
 		case V_CISCO_XR: sx_report(SX_FATAL, "unreachable point\n");
 		case V_JSON: sx_report(SX_FATAL, "unreachable point\n");
 		case V_BIRD: sx_report(SX_FATAL, "unreachable point\n");
+		case V_OPENBGPD: return bgpq3_print_openbgpd_prefixlist(f,b);
+		case V_FORMAT: sx_report(SX_FATAL, "unreachable point\n");
 	};
 	return 0;
 };
