@@ -47,6 +47,33 @@ sx_prefix_adjust_masklen(struct sx_prefix* p)
 	};
 };
 
+void
+sx_prefix_mask(struct sx_prefix* p, struct sx_prefix* q)
+{
+	int i;
+	memset(q->addr.addrs, 0, sizeof(q->addr.addrs));
+	q->family=p->family;
+	q->masklen=p->masklen;
+	for(i=0;i<p->masklen/8;i++)
+		q->addr.addrs[i]=0xff;
+	for(i=1;i<=p->masklen%8;i++)
+		q->addr.addrs[p->masklen/8]|=(1<<(8-i));
+};
+
+void
+sx_prefix_imask(struct sx_prefix* p, struct sx_prefix* q)
+{
+	int i;
+	memset(q->addr.addrs, 0xff, sizeof(q->addr.addrs));
+	q->family=p->family;
+	q->masklen=p->masklen;
+	for(i=0;i<p->masklen/8;i++)
+		q->addr.addrs[i]=0;
+	for(i=1;i<=p->masklen%8;i++)
+		q->addr.addrs[p->masklen/8]&=~(1<<(8-i));
+};
+
+
 int
 sx_prefix_parse(struct sx_prefix* p, int af, char* text)
 {
@@ -217,7 +244,7 @@ sx_prefix_range_parse(struct sx_radix_tree* tree, int af, int maxlen,
 	};
 	SX_DEBUG(debug_expander, "parsed prefix-range %s as %lu-%lu\n",
 		text, min, max);
-	if (max > maxlen) 
+	if (max > maxlen)
 		max = maxlen;
 	sx_radix_tree_insert_specifics(tree, p, min, max);
 	return 1;
@@ -264,6 +291,66 @@ sx_prefix_snprintf(struct sx_prefix* p, char* rbuffer, int srb)
 	return snprintf(rbuffer,srb,"%s/%i",buffer,p->masklen);
 };
 
+int
+sx_prefix_snprintf_fmt(struct sx_prefix* p, char* buffer, int size,
+	const char* name, const char* format)
+{
+	unsigned off=0;
+	const char* c=format;
+	struct sx_prefix q;
+	while(*c) {
+		if(*c=='%') {
+			switch(*(c+1)) {
+				case 'r':
+				case 'n':
+					inet_ntop(p->family,&p->addr,buffer+off,size-off);
+					off=strlen(buffer);
+					break;
+				case 'l':
+					off+=snprintf(buffer+off,size-off,"%i",p->masklen);
+					break;
+				case '%':
+					buffer[off++]='%';
+					break;
+				case 'N':
+					off+=snprintf(buffer+off,size-off,"%s",name);
+					break;
+				case 'm':
+					sx_prefix_mask(p, &q);
+					inet_ntop(p->family,&q.addr,buffer+off,size-off);
+					off=strlen(buffer);
+					break;
+				case 'i':
+					sx_prefix_imask(p, &q);
+					inet_ntop(p->family,&q.addr,buffer+off,size-off);
+					off=strlen(buffer);
+					break;
+				default :
+					sx_report(SX_ERROR, "Unknown format char '%c'\n", *(c+1));
+					return 0;
+			};
+			c+=2;
+		} else if (*c=='\\') {
+			switch(*(c+1)) {
+				case 'n':
+					buffer[off++]='\n'; break;
+				case 't':
+					buffer[off++]='\t'; break;
+				case '\\':
+					buffer[off++]='\\'; break;
+				default:
+					buffer[off++]=*(c+1);
+					break;
+			};
+			c+=2;
+		} else {
+			buffer[off++]=*c;
+			c++;
+		};
+	};
+	return strlen(buffer);
+};
+
 int
 sx_prefix_jsnprintf(struct sx_prefix* p, char* rbuffer, int srb)
 {
