@@ -49,6 +49,8 @@ int sx_prefix_parse(struct sx_prefix* p, int af, char* text);
 int sx_prefix_range_parse(struct sx_radix_tree* t, int af, int ml, char* text);
 int sx_prefix_fprint(FILE* f, struct sx_prefix* p);
 int sx_prefix_snprintf(struct sx_prefix* p, char* rbuffer, int srb);
+int sx_prefix_snprintf_fmt(struct sx_prefix* p, char* rbuffer, int srb,
+	const char* name, const char* fmt);
 int sx_prefix_jsnprintf(struct sx_prefix* p, char* rbuffer, int srb);
 struct sx_radix_tree* sx_radix_tree_new(int af);
 struct sx_radix_node* sx_radix_node_new(struct sx_prefix* prefix);
